# -*- coding: utf-8 -*-
from lxml.objectify import parse
from django.conf import settings
from django.utils.translation import ugettext as _
from django.utils.translation import ugettext_lazy as _l
from django.utils.translation import get_language
import re
import random


# A catalog of the institution categories. The order in which they appear
# here is the same as they will appear on the web.
institution_categories =  settings.INSTITUTION_CATEGORIES

xpath_ns = {'ds': 'http://www.w3.org/2000/09/xmldsig#',
            'md': 'urn:oasis:names:tc:SAML:2.0:metadata',
            'mdrpi': 'urn:oasis:names:tc:SAML:metadata:rpi',
            'mdui': 'urn:oasis:names:tc:SAML:metadata:ui',
            'shibmd': 'urn:mace:shibboleth:metadata:1.0',
            'xi': 'http://www.w3.org/2001/XInclude',
            'xsi': 'http://www.w3.org/2001/XMLSchema-instance'}


class ShibbolethMetadata:
    """Basic object holding the shibboleth metadata"""

    def __init__(self,filename):
        """Initialize a ShibbolethMetadata object

        arguments:
            filename -- The location of the XML document containing Shibboleth metadata

        """

        self.mdtree = parse(filename)
        self.metadata = self.mdtree.getroot()

    def getEntities(self, entity_type = None, augmented=False):
        """Returns an EntityList holding all Entities (of requested type) found
        in the metadata, perhaps augmented in to the corresponding object type
        """
        if entity_type is "idp":
            entityfilter = "IDPSSODescriptor"
        elif entity_type is "sp":
            entityfilter = "SPSSODescriptor"
        # elif entity_type is "aa":
        #     entityfilter = "AttributeAuthorityDescriptor"
        else:
            entityfilter = None

        def filtertype(entity):
            if entityfilter is None:
                return True
            elif hasattr(entity, entityfilter):
                return True
            else:
                return False

        kots = filter(filtertype, self.metadata.xpath("//md:EntityDescriptor",
                                                     namespaces=xpath_ns))

        if augmented:
            return EntityList([ Entity.construct_augmented_entity(kot)
                                for kot in kots])
        else:
            return EntityList([ Entity(kot) for kot in kots])

    def getIdps(self):
        """Returns an IdpList holding all Identity Providers found in the metadata"""
        idps = self.getEntities("idp")

        return IdpList([ IdentityProvider.create_from_entity(kot) for kot in idps ])

    def getSps(self):
        """Returns an IdpList holding all Identity Providers found in the metadata"""
        sps = self.getEntities("sp")

        return SpList([ ServiceProvider.create_from_entity(kot) for kot in sps ])

    def getPath(self, entity):
        if isinstance(entity, Entity) and hasattr(entity, 'el'):
            return self.mdtree.getpath(entity.el)
        else:
            raise Exception('Can not get path for entity %s' % str(entity))


class EntityList(list):
    """Class holding a list of SAML Entities"""

    def __init__(self,entity_list):
        """Create a new list of SAML Entities

        arguments:
            entity_list -- a list of SAML Entities

        """
        list.__init__(self,entity_list)

    def __getitem__(self, key):
        # Allow for "entitylist['a_provider_entityid'] lookups
        try:
            return filter(lambda x: x.id == key, self)[0]
        except:
            return None

    def getGroups(self):
        """Returns the list of known groups entities are grouped by (in
        EntitiesDescriptor elements)

        """
        groups = set()
        for x in self:
            groups.update(x.groups)
        return groups

    def getEntities(self, lang=None, group=None, logosize=tuple()):
        """Returns a list of entities, where each entity is represented by a
        dict carrying the localized name, url, logo and entity ID attributes

        """
        if not lang:
            lang = get_language()

        if group:
            entities = filter(lambda x: group in x.groups, self)
        else:
            entities = self

        entities = sorted(entities)

        return map(lambda x: {'name': x.getName(lang),
                              'url': x.getURL(lang),
                              'logo': x.getLogo(targetsize=logosize),
                              'id': x.id },
                   entities)

    def getEntitiesByGroup(self, lang=None, exclude=[]):
        """Returns a sequence of tuples of the form:

        (group, [ entity1, entity2, ...])

        where the second element of each tuple is the return value of
        getEntities()

        """
        groups = filter(lambda x: x not in exclude, self.getGroups())

        entities = []
        for group in groups:
            entities.append(self.getEntities(lang=lang, group=group))

        return zip(groups, entities)


class IdpList(EntityList):
    """Class holding a list of SAML Identity Providers"""

    def __init__(self,idplist):
        """Create a new list of Identity Providers

        arguments:
            idplist -- a list of IdentityProvider instances

        """
        super(EntityList, self).__init__(idplist)

    def getCategories(self):
        """Returns the list of known categories of Identity Providers"""

        return sorted(set(map(lambda x: x.getType(), self)))

    def getCategoryIdps(self, category):
        """Returns the list of known identity providers for a given category"""

        return filter(lambda x: x.getType() == category, self)

    def getIdpForScope(self, scope):
        """Returns the identity provider matching a given scope"""

        try:
            return filter(lambda x: x.matchesScope(scope), self)[0]
        except:
            return None

    def getIdpsByCategory(self, lang=None, exclude=None):
        """Returns a sequence of tuples of the form:

        (category, [ idp1, idp2, ...])

        where idpX is a dict { 'name': name, 'id': entityid }

        The list of idps is sorted by name

        """
        if not lang:
            lang = get_language()

        if exclude:
            validcats = filter(lambda x: x[0] not in exclude, institution_categories)
        else:
            validcats = institution_categories

        cats = map(lambda x: x[0], validcats)

        categories = map(lambda x: { 'id': x[0], 'name': _l(x[1]) }, validcats)

        idps = []

        # if no valid categories are found, return all idp's
        if not validcats:
            catidps = self.getEntities()
            return [('institutions', catidps)]

        for category in cats:
            catidps = sorted(self.getCategoryIdps(category))
            idps.append(map(lambda x: {
                    'name': x.getName(lang),
                    'url': x.getURL(lang),
                    'id': x.id },
                    catidps))

        return zip(categories, idps)


class SpList(EntityList):
    """Class holding a list of SAML Service Providers"""

    def __init__(self,splist):
        """Create a new list of Service Providers

        arguments:
            splist -- a list of ServiceProvider instances

        """
        super(EntityList, self).__init__(splist)


class Entity:
    """Basic class holding a SAML Entity"""

    def __init__(self,el):
        """Create a new Entity instance
        arguments:
            el -- An lxml.objectify.Element holding an EntityDescriptor for a SAML Entity

        """

        self.el = el
        self.name = {} # Holds the Entity's name in a form { language: string }
        self.url = {}
        self.logo = {}
        self.id = self.el.get('entityID')
        self.groups = set(el.xpath("ancestor::md:EntitiesDescriptor/@Name", namespaces=xpath_ns))

        # Initialize the contact details
        self.contact = { 'givenName': '', 'surName': '', 'company': '', 'email': '', 'telephone': '', 'url': '' }

        # Get the institution's name
        try:
            for name in self.el.Organization.OrganizationDisplayName:
                self.name[name.get('{http://www.w3.org/XML/1998/namespace}lang')] = name.text.strip()
        except:
            self.name = {'en': "no name"}

        try:
            for url in self.el.Organization.OrganizationURL:
                self.url[url.get('{http://www.w3.org/XML/1998/namespace}lang')] = url.text.strip()
        except:
            pass

        # Fill in the contact details
        try:
            for contact in self.el.ContactPerson:
                if contact.get('contactType') == "support":
                    # We're not sure these details even exists, but since that would
                    # require a set of nested checks, exception catching is more
                    # clean.
                    try:
                        self.contact['email'] = contact.EmailAddress.text.strip()
                    except:
                        pass

                    try:
                        self.contact['telephone'] = contact.TelephoneNumber.text.strip()
                    except:
                        pass
        except:
            pass

    def __cmp__ (self,other):
        # Alphabetic sorting by name
        return cmp(self.getName(get_language()), other.getName(get_language()))

    def __repr__(self):
        return "Entity: \"" + self.name['en'] + '" (' + self.id + ')'

    @classmethod
    def create_from_entity(cls, entity):
        if issubclass(cls, entity.__class__) and hasattr(entity, 'el'):
            return cls(entity.el)
        else:
            raise Exception('Can not instantiate %s from %s' % (cls,
                                                                str(entity)))

    @staticmethod
    def construct_augmented_entity(el):
        elcls = el.__class__
        # should do isinstance() but we do not import whole lxl.objectify
        if ("%s.%s" % (elcls.__module__, elcls.__name__)) == \
                "lxml.objectify.ObjectifiedElement":
            if hasattr(el, "IDPSSODescriptor"):
                aug_ent_cls = IdentityProvider
            elif hasattr(el, "SPSSODescriptor"):
                aug_ent_cls = ServiceProvider
            # elif hasattr(el, "AttributeAuthorityDescriptor"):
            #     pass
            else:
                aug_ent_cls = None

            if aug_ent_cls is not None:
                return aug_ent_cls(el)

        raise Exception("Can not instantiate augmented Entity from %s" %
                             el)

    def getName(self,lang=None):
        if not lang:
            lang = get_language()

        try:
            return self.name[lang]
        except:
            try:
                return self.name['en']
            except:
                return self.name[self.name.keys()[0]]

    def getURL(self,lang=None):
        if not lang:
            lang = get_language()

        try:
            return self.url[lang]
        except:
            pass

        try:
            return self.url['en']
        except:
            pass

        return None

    def getLogo(self,targetsize=tuple()):

        if not self.logo:
            return None

        # get (x, y) closest to target (a, b)
        if isinstance(targetsize, tuple) and \
                len(targetsize) is 2 and \
                [isinstance(i, int) for i in targetsize]:
            dimensions = min(self.logo.keys(),
                             key = lambda x: abs(x[0]-targetsize[0]) + \
                                 abs(x[1]-targetsize[1])
                             )
        else:
            dimensions = random.choice(self.logo.keys())

        return { 'width': dimensions[0],
                 'height': dimensions[1],
                 'data': self.logo[dimensions] }


class IdentityProvider(Entity):
    """Class holding a SAML Identity Provider"""

    def __init__(self,idpel):
        """Create a new IdentityProvider instance
        arguments:
            idpel -- An lxml.objectify.Element holding an EntityDescriptor for a shibboleth IdP

        """

        Entity.__init__(self, idpel)

        # Dictionary to hold all ArtifactResolutionService definitions, by Binding
        self.ars = {}
        # Dictionary to hold all SingleLogoutService definitions, by Binding
        self.slo = {}
        # Dictionary to hold all ManageNameIDService definitions, by Binding
        self.mnid = {}
        # Dictionary to hold all SingleSignOnService definitions, by Binding
        self.sso = {}

        # Get all SingleSignOnService definitions (required)
        for entry in self.el.IDPSSODescriptor.SingleSignOnService:
            self.sso[entry.get('Binding')] = entry.get('Location')
        try:
            # Get all SingleLogoutService definitions
            for entry in self.el.IDPSSODescriptor.SingleLogoutService:
                self.slo[entry.get('Binding')] = entry.get('Location')
            # Get all ArtifactResolutionService definitions
            for entry in self.el.IDPSSODescriptor.ArtifactResolutionService:
                self.ars[entry.get('Binding')] = entry.get('Location')
            # Get all ManageNameIDService definitions
            for entry in self.el.IDPSSODescriptor.ManageNameIDService:
                self.mnid[entry.get('Binding')] = entry.get('Location')
        except:
            pass

        # Override self.name, self.url with mdui:DisplayName, mdui:DisplayName
        # also add logo from mdui:Logo
        try:
            ui = self.el.IDPSSODescriptor.\
                Extensions['{urn:oasis:names:tc:SAML:metadata:ui}UIInfo']
            # Override self.name with mdui:DisplayName or self.atcs.name
            if hasattr(ui, 'DisplayName'):
                self.name = {}
                for name in ui.DisplayName:
                    self.name[name.get(
                            '{http://www.w3.org/XML/1998/namespace}lang'
                            )] = name.text
            # Override self.url with mdui:InformationURL
            if hasattr(ui,'InformationURL'):
                self.url = {}
                for url in ui.InformationURL:
                    self.url[url.get(
                            '{http://www.w3.org/XML/1998/namespace}lang'
                            )] = url.text
            # Also add logo from mdui:Logo
            if hasattr(ui, 'Logo'):
                for logo in ui.Logo:
                    self.logo[(int(logo.get('width')),
                               int(logo.get('height')))] = logo.text
        except:
            pass

    def __repr__(self):
        return "IDP: \"" + self.name['en'] + '" (' + self.id + ')'

    def getType(self):
        """Returns the type (category) of the current IdP"""

        # Some heuristics to determine the IdP type, based on the
        # institution's name in english.
        for lang in self.name.values():
            if lang.lower().find('test') >= 0:
                return "test"
            elif re.findall(r'(univerisyt|universiteit|universite|school of fine arts)', lang.lower()):
                return "university"
            elif lang.lower().find('technological') >= 0:
                return "tei"
            elif re.findall(r'(ecclesiastical|school|academy)', lang.lower()):
                return "school"
            elif re.findall(r'(institute|cent(er|re)|ncsr|foundat|bservat)', lang.lower()):
                return "institute"
        return "other"

    def getScope(self):
        """Returns the scope of the current IdP"""

        scopes = filter(lambda x: x.tag == "{urn:mace:shibboleth:metadata:1.0}Scope", self.el.IDPSSODescriptor.Extensions.getchildren())
        return scopes[0].text

    def matchesScope(self,scope):
        """Checks wheter the current IdPs scope matches the given string"""

        myscope = self.getScope()

        # Append a trailing '$', to align the regexp with the string end
        if myscope[-1] != '$':
            myscope += '$'

        if re.search(myscope, scope):
            return True

        return False


class ServiceProvider(Entity):
    """Class holding a SAML Service Provider"""

    def __init__(self,spel):
        """Create a new ServiceProvider instance
        arguments:
            spel -- An lxml.objectify.Element holding an EntityDescriptor for a shibboleth SP

        """

        Entity.__init__(self, spel)

        # Dictionary to hold all AssertionConsumerService definitions, by Binding
        self.acs = {}
        # Dictionary to hold all SingleLogoutService definitions, by Binding
        self.slo = {}
        # Dictionary to hold all ManageNameIDService definitions, by Binding
        self.mnid = {}
        # Dictionary to hold all AttributeConsumingService definitions, by ServiceName
        self.atcs = {}

        # Get all AssertionConsumerService definitions (required)
        for entry in self.el.SPSSODescriptor.AssertionConsumerService:
            self.acs[entry.get('Binding')] = entry.get('Location')
        # Get all SingleLogoutService definitions
        try:
            for entry in self.el.SPSSODescriptor.SingleLogoutService:
                self.slo[entry.get('Binding')] = entry.get('Location')
        except:
            pass
        # Get all ManageNameIDService definitions
        try:
            for entry in self.el.SPSSODescriptor.ManageNameIDService:
                self.mnid[entry.get('Binding')] = entry.get('Location')
        except:
            pass
        # Get all AttributeConsumingService definitions
        try:
            for entry in self.el.SPSSODescriptor.AttributeConsumingService:
                atcs_name = {}
                if hasattr(entry, 'ServiceName'):
                    for name in entry.ServiceName:
                        atcs_name[name.get(
                                '{http://www.w3.org/XML/1998/namespace}lang'
                                )] = name.text
                atcs_descr = {}
                if hasattr(entry, 'ServiceDescription'):
                    for descr in entry.ServiceDescription:
                        atcs_descr[descr.get(
                                '{http://www.w3.org/XML/1998/namespace}lang'
                                )] = descr.text
                req_attr = {}
                if hasattr(entry, 'RequestedAttribute'):
                    for attr in entry.RequestedAttribute:
                        req_attr[(attr.get('Name'),
                                  attr.get('NameFormat'))] = {
                            'name': attr.get('FriendlyName'),
                            'required':
                                True if \
                                attr.get('isRequired').lower == 'true' \
                                else False
                            }
                if 'en' in atcs_name:
                    self.atcs[atcs_name['en']] = type(
                        self.__class__.__name__ + \
                            '.AttributeConsumingService',
                        (object,),
                        { 'name': atcs_name,
                          'descr': atcs_descr,
                          'attr': req_attr }
                        )
        except AttributeError:
            pass

        # Override self.name with self.atcs.name
        if self.atcs:
            self.name = [s.name for s in self.atcs.values()][0]

        try:
            ui = self.el.SPSSODescriptor.\
                Extensions['{urn:oasis:names:tc:SAML:metadata:ui}UIInfo']
            # Override self.name with mdui:DisplayName
            if hasattr(ui, 'DisplayName'):
                self.name = {}
                for name in ui.DisplayName:
                    self.name[name.get(
                            '{http://www.w3.org/XML/1998/namespace}lang'
                            )] = name.text
            # Override self.url with mdui:InformationURL
            if hasattr(ui,'InformationURL'):
                self.url = {}
                for url in ui.InformationURL:
                    self.url[url.get(
                            '{http://www.w3.org/XML/1998/namespace}lang'
                            )] = url.text
            # Also add logo from mdui:Logo
            if hasattr(ui, 'Logo'):
                for logo in ui.Logo:
                    self.logo[(int(logo.get('width')),
                               int(logo.get('height')))] = logo.text
        except AttributeError:
            pass

    def __repr__(self):
        return "SP: \"" + self.name['en'] + '" (' + self.id + ')'



from django.conf import settings
from django.conf.urls import patterns, url, include

urlpatterns = []

urlpatterns += patterns('',
    (r'^/?$', 'wayf.views.wayf'),
    (r'^setlanguage/(.*)', 'wayf.views.setlanguage'),
)



from wayf.utils import ShibbolethMetadata
# optional for now , getUserRealm
import time
from django.shortcuts import render_to_response
from django.conf import settings
from django.http import HttpResponseRedirect
from django.utils.http import urlencode
from django.template import RequestContext


def wayf(request):
    # Instantiate the metadata
    metadata = ShibbolethMetadata(settings.SHIB_METADATA)

    # Get the IdP list
    idps = metadata.getIdps()

    # A list to hold the cookies-to-be-set
    cookies = []

    # Get the current IdP, if there is one
    if settings.IDP_COOKIE in request.COOKIES.keys():
        current_idp = idps[request.COOKIES[settings.IDP_COOKIE]]
    else:
        current_idp = None

    # Try to get the user's last used IdP
    if settings.LAST_IDP_COOKIE in request.COOKIES.keys():
        selectedidp = idps[request.COOKIES[settings.LAST_IDP_COOKIE]]
    else:
        selectedidp = None

    # If this is the first visit, use some IP-based heuristics, as in utils.py's getUserRealm()
    # this uses the getUserRealm magic from dnsutils
    #if not selectedidp:
    #    selectedidp = idps.getIdpForScope(getUserRealm(request.META['REMOTE_ADDR']))

    # First check to see if anything has changed
    if request.method == "POST":
        if 'clear' in request.POST.keys():
            if request.POST['clear']:
                response = HttpResponseRedirect("/")
                response.delete_cookie(settings.IDP_COOKIE, domain=settings.COOKIE_DOMAIN)
                response['P3P'] = settings.P3P_HEADER
                return response

        elif 'user_idp' in request.POST.keys():
            current_idp = idps[request.POST['user_idp']]
            if current_idp:
                cookies.append({'name': settings.LAST_IDP_COOKIE, 'data': request.POST['user_idp'], 'age': 86400 * 100})
                if request.POST.get('save'):
                    if request.POST.get('savetype') == 'perm':
                        age = 86400 * 100
                    else:
                        age = None
                    cookies.append({'name': settings.IDP_COOKIE, 'data': request.POST['user_idp'], 'age': age })

    # At this point we have handled the cookies and have an IdP, if the intent is such
    if not request.GET:
        # We were called without any arguments
        if current_idp:
            response = render_to_response("wayf_set.html", { 'currentidp': current_idp.getName() }, context_instance=RequestContext(request))
            for cookie in cookies:
                if cookie['age']:
                    expires = time.strftime("%a, %d-%m-%y %H:%M:%S GMT", time.gmtime(time.time() + cookie['age']))
                else:
                    expires = None
                response.set_cookie(cookie['name'], cookie['data'], domain=settings.COOKIE_DOMAIN, max_age=cookie['age'], expires = expires)
        else:
            idplist = idps.getIdpsByCategory()
            response = render_to_response("wayf.html", { 'idplist': idplist, 'request': request, 'selected': selectedidp }, context_instance=RequestContext(request))

        response['P3P'] = settings.P3P_HEADER
        return response

    # If we got to this point, then this is a request comming from an SP
    if current_idp:
        # We have an IdP to route the request to
        if 'entityID' in request.GET.keys() and 'return' in request.GET.keys():
            # a SAML Discovery Service request
            # Discovery Service mandates that 'entityID' holds the SP's ID
            if 'returnIDParam' in request.GET.keys() and request.GET['returnIDParam']:
                returnparam = request.GET['returnIDParam']
            else:
                returnparam = 'entityID'

            response = HttpResponseRedirect(request.GET['return'] + "&" + urlencode(((returnparam, current_idp.id),)))
        elif 'shire' in request.GET.keys() and 'target' in request.GET.keys():
            # an old Shibboleth 1.x request
            # We just redirect the user to the given IdP
            response = HttpResponseRedirect(
                current_idp.sso['urn:mace:shibboleth:1.0:profiles:AuthnRequest'] + "?" + request.GET.urlencode()
                )
        else:
            response = render_to_response("500.html")
            response.status_code = 400 # bad request

        for cookie in cookies:
            if cookie['age']:
                expires = time.strftime("%a, %d-%m-%y %H:%M:%S GMT", time.gmtime(time.time() + cookie['age']))
            else:
                expires = None
            response.set_cookie(cookie['name'], cookie['data'], domain=settings.COOKIE_DOMAIN, max_age=cookie['age'], expires = expires)

        response['P3P'] = settings.P3P_HEADER
        return response


    # If we got this far, then we need to be redirected, but don't know where to.
    # Let the user pick an IdP
    # Generate the category - idp list
    idplist = idps.getIdpsByCategory()

    # Render the apropriate wayf template
    response = render_to_response("wayf_from_sp.html", { 'idplist': idplist, 'request': request, 'selected': selectedidp }, context_instance=RequestContext(request) )
    response['P3P'] = settings.P3P_HEADER
    return response

def index(request):
    return render_to_response("index.html", context_instance=RequestContext(request))

def idp_list(request):
    metadata = ShibbolethMetadata(settings.SHIB_METADATA)
    idps = metadata.getIdps()
    idplist = idps.getIdpsByCategory(exclude=('wayf', 'test'))

    return render_to_response("idp_list.html", { 'idplist' : idplist }, context_instance=RequestContext(request) )

def sp_list(request):
    metadata = ShibbolethMetadata(settings.SHIB_METADATA)
    sps = metadata.getSps()
    splist = sps.getEntitiesByGroup()
    # splist_other = entities in the top group
    splist_other = [i[1] for i in splist if i[0] == 'http://www.grnet.gr/aai'][0]
    # fitlerids = entity.id for entities not in the top group
    filterids = [o['id'] for i in splist for o in i[1] if i[0] != 'http://www.grnet.gr/aai']
    # filter out entities not in the top group from splist_other
    splist_other_new = filter(lambda x: x['id'] not in filterids, splist_other)
    # replace top group with filtered out version in splist
    splist.insert(splist.index(('http://www.grnet.gr/aai', splist_other)), ('other', splist_other_new))
    splist.remove(('http://www.grnet.gr/aai', splist_other))

    return render_to_response("sp_list.html", { 'splist' : splist }, context_instance=RequestContext(request) )

def entity_list(request, group = None):
    if group is not None:
        group = "http://aai.grnet.gr%s" % request.path_info
    metadata = ShibbolethMetadata(settings.SHIB_METADATA)
    entities = metadata.getEntities(augmented=True)
    entlist = entities.getEntities(group=group, logosize=(100,100))

    return render_to_response("entity_list.html", { 'entlist' : entlist,
                                                    'group' : group } , context_instance=RequestContext(request))

""" example support view
uses urldecode from dnsutils
and needs an idpmap from somewhere
def support(request, mode="support"):
    # This gets triggered when a user's attributes fail to be accepted
    # by a service provider. The aim is to produce a help page, indicating
    # the user's home institution contact details.

    opts = {}
    userIdp = None

    # Check to see whether _redirect_user_idp is set. This cookie will include
    # The user's selected IdP
    if settings.IDP_COOKIE in request.COOKIES.keys():
        userIdp = urldecode(request.COOKIES[settings.IDP_COOKIE])
    elif settings.LAST_IDP_COOKIE in request.COOKIES.keys():
        userIdp = urldecode(request.COOKIES[settings.LAST_IDP_COOKIE])

    if userIdp:
        # Check to see if this is one of the old WAYF entries and map it to a
        # new entityID instead.
        if userIdp in idpmap.keys():
            userIdp = idpmap[userIdp]

        # Get the corresponding IdentityProvider instance
        idp = ShibbolethMetadata(settings.SHIB_METADATA).getIdps()[userIdp]

        if idp:
            opts['idp'] = idp
            opts['idpname'] = idp.getName()

    if mode == "help":
        response = render_to_response("help.html", opts, context_instance=RequestContext(request))
    else:
        response = render_to_response("support.html", opts, context_instance=RequestContext(request))

    response['P3P'] = 'CP="NOI CUR DEVa OUR IND COM NAV PRE"'
    return response
"""

def setlanguage(request, lang):
    try:
        url = request.META['HTTP_REFERER']
    except KeyError:
        url = '/'

    response = HttpResponseRedirect(url)
    response.set_cookie(settings.LANGUAGE_COOKIE_NAME, lang, domain=settings.COOKIE_DOMAIN, max_age = 100 * 86400, expires = time.strftime("%a, %d-%m-%y %H:%M:%S GMT", time.gmtime(time.time() + 100 * 86400)))
    response['P3P'] = settings.P3P_HEADER
    return response



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-wayf',
    version='0.2.0',
    packages=find_packages(),
    include_package_data=True,
    license='GPL v3',
    description='A simple Django implementation of a shibboleth/saml wayf service',
    long_description=README,
    url='https://github.com/hpcugent/django-wayf',
    author='Jens Timmerman',
    author_email='jens.timmerman@ugent.be',
    install_requires=['lxml'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    zip_safe=False,
)
